from pytest_shell.dialect import BashDialect, Dialect


class ShellSession(Dialect):
    def __call__(self, envvars=None, source=None, pwd=None):
        ...

    def run_script(self, path, args=None, timeout=10.0, soft_timeout=True):
        ...


class LocalBashSession(ShellSession, BashDialect):
    pass



from _typeshed import Incomplete
from xmldiff import diff as diff
from xmldiff import formatting as formatting
from xmldiff import patch as patch

FORMATTERS: Incomplete

def diff_trees(
    left,
    right,
    diff_options: Incomplete | None = ...,
    formatter: Incomplete | None = ...,
): ...
def diff_texts(
    left,
    right,
    diff_options: Incomplete | None = ...,
    formatter: Incomplete | None = ...,
): ...
def diff_files(
    left,
    right,
    diff_options: Incomplete | None = ...,
    formatter: Incomplete | None = ...,
): ...
def make_diff_parser(): ...
def diff_command(args: Incomplete | None = ...) -> None: ...
def patch_tree(actions, tree): ...
def patch_text(actions, tree): ...
def patch_file(actions, tree): ...
def make_patch_parser(): ...
def patch_command(args: Incomplete | None = ...) -> None: ...



from collections.abc import Generator

from _typeshed import Incomplete

def post_order_traverse(node) -> Generator[Incomplete, None, None]: ...
def reverse_post_order_traverse(node) -> Generator[Incomplete, None, None]: ...
def breadth_first_traverse(node) -> Generator[Incomplete, None, None]: ...
def longest_common_subsequence(left_sequence, right_sequence, eqfn=...): ...

WHITESPACE: Incomplete

def cleanup_whitespace(text): ...
def getpath(element, tree: Incomplete | None = ...): ...
def make_ascii_tree(element): ...



import os
import re
import tempfile
from subprocess import PIPE, Popen
from typing import List

from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor

MERMAID_START = re.compile(r"^```[\ \t]*[Mm]ermaid[\ \t]*$")
MERMAID_END = re.compile(r"^```[\ \t]*$")
CONF_FILE = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "mermaid.json"
)


def generate_image(mermaid_code: str) -> str:
    """
    Converts Mermaid source code to SVG.
    :param mermaid_code: Mermaid input code as a string.
    :return: SVG code as a string.
    """
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.write(mermaid_code.encode("utf8"))
    tf.flush()

    name = f"{tf.name}.svg"
    cmdline = [
        "npm",
        "exec",
        "--package=@mermaid-js/mermaid-cli",
        "--",
        "mmdc",
        "-i",
        tf.name,
        "-o",
        name,
        "-c",
        CONF_FILE,
    ]

    try:
        p = Popen(cmdline, stdout=PIPE, stderr=PIPE)
        out, err = p.communicate()
    except Exception as exc:
        raise Exception("Failed to run mermaid: %s" % exc)
    else:
        if p.returncode == 0:
            os.remove(tf.name)
            with open(name) as f:
                svg_string = f.read()
            os.remove(name)
            return svg_string
        else:
            raise RuntimeError(
                f"Error calling mermaid: {str(err)}\nTemp files: {tf.name} and {name}\nCommand: {' '.join(cmdline)}"
            )


class MermaidProcessor(Preprocessor):
    def run(self, lines: List[str]) -> List[str]:
        def gen():
            looking_for_mermaids = True
            mermaid_code = ""

            for line in lines:
                # Ad hoc state machine to the rescue! (We only have two states:
                # either we're waiting for a potential Mermaid block or we're in
                # the middle of processing one).
                if looking_for_mermaids:
                    if MERMAID_START.match(line):
                        # Caught a Mermaid! Switch state so the other other state
                        # can start slurping the Mermaid code.
                        looking_for_mermaids = False
                        continue
                    else:
                        # No mermaids sighted yet. Leave alone what's not ours
                        # to touch.
                        yield line
                else:
                    if MERMAID_END.match(line):
                        # Finished gobbling code. Convert to SVG, yield the SVG code
                        # instead of the Mermaid source we've been gobbling.
                        yield generate_image(mermaid_code)
                        # Reset the Mermaid code buffer and start looking for
                        # more mermaids in the same document...
                        mermaid_code = ""
                        looking_for_mermaids = True
                        continue
                    else:
                        # Slurp up that Mermaid code and gobble[1] the lines so they
                        # get removed from the source Markdown.
                        # [1] i.e. don't yield the lines
                        mermaid_code += line + "\n"
                        continue

        return list(gen())


class MermaidExtension(Extension):
    def extendMarkdown(self, md):
        md.preprocessors.register(MermaidProcessor(), "mermaid", 35)
        md.registerExtension(self)


def makeExtension(**kwargs):  # pragma: no cover
    return MermaidExtension(**kwargs)



from collections.abc import Generator
from typing import NamedTuple

from _typeshed import Incomplete
from xmldiff import utils as utils
from xmldiff.diff_match_patch import diff_match_patch as diff_match_patch

DIFF_NS: str
DIFF_PREFIX: str
INSERT_NAME: Incomplete
DELETE_NAME: Incomplete
RENAME_NAME: Incomplete
WS_BOTH: int
WS_TEXT: int
WS_TAGS: int
WS_NONE: int
T_OPEN: int
T_CLOSE: int
T_SINGLE: int
PLACEHOLDER_START: int

class BaseFormatter:
    def __init__(self, normalize=..., pretty_print: bool = ...) -> None: ...
    def prepare(self, left_tree, right_tree) -> None: ...
    def format(self, diff, orig_tree) -> None: ...

class PlaceholderEntry(NamedTuple):
    element: Incomplete
    ttype: Incomplete
    close_ph: Incomplete

class PlaceholderMaker:
    text_tags: Incomplete
    formatting_tags: Incomplete
    placeholder2tag: Incomplete
    tag2placeholder: Incomplete
    placeholder: Incomplete
    diff_tags: Incomplete
    def __init__(self, text_tags=..., formatting_tags=...) -> None: ...
    def get_placeholder(self, element, ttype, close_ph): ...
    def is_placeholder(self, char): ...
    def is_formatting(self, element): ...
    def do_element(self, element) -> None: ...
    def do_tree(self, tree) -> None: ...
    def split_string(self, text): ...
    def undo_string(self, text): ...
    def undo_element(self, elem) -> None: ...
    def undo_tree(self, tree) -> None: ...
    def mark_diff(self, ph, action): ...
    def wrap_diff(self, text, action): ...

class XMLFormatter(BaseFormatter):
    normalize: Incomplete
    pretty_print: Incomplete
    text_tags: Incomplete
    formatting_tags: Incomplete
    placeholderer: Incomplete
    def __init__(
        self,
        normalize=...,
        pretty_print: bool = ...,
        text_tags=...,
        formatting_tags=...,
    ) -> None: ...
    def prepare(self, left_tree, right_tree) -> None: ...
    def finalize(self, result_tree) -> None: ...
    def format(self, diff, orig_tree, differ: Incomplete | None = ...): ...
    def render(self, result): ...
    def handle_action(self, action, result) -> None: ...

class DiffFormatter(BaseFormatter):
    normalize: Incomplete
    def __init__(self, normalize=..., pretty_print: bool = ...) -> None: ...
    def prepare(self, left, right) -> None: ...
    def finalize(self, left, right) -> None: ...
    def format(self, diff, orig_tree): ...
    def handle_action(self, action): ...

class XmlDiffFormatter(BaseFormatter):
    normalize: Incomplete
    def __init__(self, normalize=..., pretty_print: bool = ...) -> None: ...
    def prepare(self, left, right) -> None: ...
    def finalize(self, left, right) -> None: ...
    def format(self, diff, orig_tree): ...
    def handle_action(
        self, action, orig_tree
    ) -> Generator[Incomplete, None, None]: ...



from textwrap import dedent

from xmldiff import main

from voltaire import graphviz


def test_generate_image():
    dot = """
    digraph g {
        A -> B;
    }
    """
    svg = graphviz.generate_image(dot)
    assert "DOCTYPE" not in svg
    expected = dedent(
        """\
        <svg width="62pt" height="116pt"
         viewBox="0.00 0.00 62.00 116.00" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
        <g id="graph0" class="graph" transform="scale(1 1) rotate(0) translate(4 112)">
        <title>g</title>
        <polygon fill="white" stroke="transparent" points="-4,4 -4,-112 58,-112 58,4 -4,4"/>
        <!-- A -->
        <g id="node1" class="node">
        <title>A</title>
        <ellipse fill="none" stroke="black" cx="27" cy="-90" rx="27" ry="18"/>
        <text text-anchor="middle" x="27" y="-86.3" font-family="Times,serif" font-size="14.00">A</text>
        </g>
        <!-- B -->
        <g id="node2" class="node">
        <title>B</title>
        <ellipse fill="none" stroke="black" cx="27" cy="-18" rx="27" ry="18"/>
        <text text-anchor="middle" x="27" y="-14.3" font-family="Times,serif" font-size="14.00">B</text>
        </g>
        <!-- A&#45;&gt;B -->
        <g id="edge1" class="edge">
        <title>A&#45;&gt;B</title>
        <path fill="none" stroke="black" d="M27,-71.7C27,-63.98 27,-54.71 27,-46.11"/>
        <polygon fill="black" stroke="black" points="30.5,-46.1 27,-36.1 23.5,-46.1 30.5,-46.1"/>
        </g>
        </g>
        </svg>
    """
    )
    print(svg)
    diff = main.diff_texts(svg.encode("utf8"), expected.encode("utf8"))
    assert diff == []



import os

from bs4 import BeautifulSoup
from pelican import signals


def words(page):
    page_text = BeautifulSoup(page.content, "html.parser").getText()
    return len(page_text.split())


class StatsGenerator:
    def __init__(self, context, output_path, *args, **kwargs):
        self.context = context
        self.output_path = output_path

    def generate_output(self, writer):
        pages = self.context["pages"] + self.context["articles"]

        total = 0

        for page in pages:
            total += words(page)

        path = os.path.join(self.output_path, "words.txt")
        with open(path, "w", encoding="utf-8") as fd:
            fd.write(f"{total}\n")


def get_generators(_generators):
    return StatsGenerator


def register():
    signals.get_generators.connect(get_generators)



from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md")) as f:
    long_description = f.read()

setup(
    name="voltaire",
    use_scm_version={
        "local_scheme": "dirty-tag",
        "write_to": "voltaire/_version.py",
        "fallback_version": "0.0.0",
    },
    description="Tools for prolific writers (or more accurately, those that want to be).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Ross Fenning",
    author_email="github@rossfenning.co.uk",
    url="https://github.com/avengerpenguin/voltaire",
    packages=["voltaire"],
    include_package_data=True,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Internet :: WWW/HTTP",
    ],
    project_urls={
        "Issue Tracker": "https://github.com/avengerpenguin/voltaire/issues",
    },
    keywords=["pelican", "invoke"],
    install_requires=[
        "invoke",
        "pelican",
        "markdown",
        "graphviz",
        "pelican-webassets",
        "pelican-sitemap",
        "plantuml-markdown",
        "cssmin",
        "livereload",
        "ghp-import",
        "beautifulsoup4",
        "typogrify",
        "webassets",
        "sh",
        "doctrine",
    ],
    setup_requires=[
        "pytest-runner",
        "setuptools_scm>=3.3.1",
        "pre-commit",
    ],
    extras_require={
        "test": [
            "pytest",
            "pytest-shell",
            "pytest-shell-utilities",
            "pytest-pikachu",
            "pytest-mypy",
            "xmldiff",
            "types-markdown",
        ],
    },
)
